#ifndef MY_STRING_HPP
#define MY_STRING_HPP

#include <iostream>
#include <cstring>

class my_string
{
private:
    struct StringData
    {
        char *data;
        int ref_count;

        StringData(const char *str) : ref_count(1)
        {
            data = new char[std::strlen(str) + 1];
            std::strcpy(data, str);
        }

        ~StringData()
        {
            delete[] data;
        }
    };

    StringData *stringData;

    void detach()
    {
        if (stringData->ref_count == 1)
        {
            return; // Already unique
        }
        stringData->ref_count--;
        stringData = new StringData(stringData->data); // Create a deep copy
    }

public:
    // Default constructor
    my_string() : stringData(new StringData("")) {}

    // Constructor with a C-string
    my_string(const char *str) : stringData(new StringData(str)) {}

    // Copy constructor
    my_string(const my_string &s) : stringData(s.stringData)
    {
        stringData->ref_count++;
    }

    // Assignment operator
    my_string &operator=(const my_string &s)
    {
        if (this != &s)
        {
            if (--stringData->ref_count == 0)
            {
                delete stringData;
            }
            stringData = s.stringData;
            stringData->ref_count++;
        }
        return *this;
    }

    // Destructor
    ~my_string()
    {
        if (--stringData->ref_count == 0)
        {
            delete stringData;
        }
    }

    // Get a character at a given index
    char getChar(const int &i) const
    {
        return stringData->data[i];
    }

    // Set a character at a given index
    void setChar(const int &i, const char &c)
    {
        detach();
        stringData->data[i] = c;
    }

    // Print the string
    void print() const
    {
        std::cout << stringData->data << std::endl;
    }
};

#endif
