#include "ref_counted.hpp"
#include "point.hpp"
#include "my_string.hpp"

int main()
{
    std::cout << "--- Testing ref_counted with Point ---" << std::endl;
    Point *p1 = new Point(1, 2);
    ref_counted<Point> rc1(p1);

    std::cout << "Initial reference count of rc1: " << rc1.count() << std::endl;
    rc1.get()->print();

    {
        ref_counted<Point> rc2 = rc1;
        std::cout << "Reference count after creating rc2: " << rc1.count() << std::endl;
        rc2.get()->print();
    }

    std::cout << "Reference count after rc2 goes out of scope: " << rc1.count() << std::endl;
    rc1.get()->print();

    std::cout << "\n--- Testing ref_counted with my_string ---" << std::endl;
    my_string s("Hello world");
    s.print();

    {
        my_string t = s;
        std::cout << "\nAfter copying s to t:" << std::endl;
        t.print();
        s.print();

        std::cout << "\nModifying t:" << std::endl;
        t.setChar(0, 'h');
        t.print();
        s.print();
    }

    std::cout << "\nAfter t goes out of scope:" << std::endl;
    s.print();

    std::cout << "\n--- End of Program ---" << std::endl;
    return 0;
}
