#ifndef REF_COUNTED_HPP
#define REF_COUNTED_HPP

#include <iostream>

template <typename T>
class ref_counted
{
private:
    T *object;      // Pointer to the managed object
    int *ref_count; // Pointer to the reference count

public:
    // Constructor
    explicit ref_counted(T *obj) : object(obj), ref_count(new int(1)) {}

    // Copy constructor
    ref_counted(const ref_counted &other) : object(other.object), ref_count(other.ref_count)
    {
        (*ref_count)++;
    }

    // Assignment operator
    ref_counted &operator=(const ref_counted &other)
    {
        if (this != &other)
        {
            if (--(*ref_count) == 0)
            {
                delete object;
                delete ref_count;
            }
            object = other.object;
            ref_count = other.ref_count;
            (*ref_count)++;
        }
        return *this;
    }

    // Destructor
    ~ref_counted()
    {
        if (--(*ref_count) == 0)
        {
            delete object;
            delete ref_count;
        }
    }

    // Access the managed object
    T *get() const { return object; }

    // Access the reference count
    int count() const { return *ref_count; }
};

#endif
