#ifndef MY_STRING_HPP
#define MY_STRING_HPP

#include "../common/ref_counted.hpp"
#include <cstring>
#include <iostream>

class my_string
{
private:
    ref_counted<char> stringData;

public:
    // Constructor with C-string input
    my_string(const char *str)
        : stringData(new char[std::strlen(str) + 1])
    {
        std::strcpy(stringData.get(), str);
    }

    // Copy constructor
    my_string(const my_string &s) : stringData(s.stringData) {}

    // Assignment operator
    my_string &operator=(const my_string &s)
    {
        stringData = s.stringData;
        return *this;
    }

    // Print the string with reference count
    void print() const
    {
        std::cout << stringData.get() << " [" << stringData.count() << "]" << std::endl;
    }

    // Access a character by index
    char getChar(int i) const
    {
        return stringData.get()[i];
    }

    // Modify a character by index with copy-on-write logic
    void setChar(int i, char c)
    {
        if (stringData.count() > 1)
        {
            char *newString = new char[std::strlen(stringData.get()) + 1];
            std::strcpy(newString, stringData.get());
            stringData = ref_counted<char>(newString); // Assign the deep copy
        }
        stringData.get()[i] = c;
    }
};

#endif
