#include "bump_allocator.hpp"
#include <iostream>

struct Data {
    int x;
    double y;
};

int main() {
    // Create an allocator with 64 bytes of memory
    BumpAllocator allocator(64);

    allocator.printStatus();

    // Allocate an integer
    int* intPtr = allocator.alloc<int>();
    if (intPtr) {
        *intPtr = 42;
        std::cout << "Allocated int: " << *intPtr << "\n";
    }

    // Allocate a struct
    Data* dataPtr = allocator.alloc<Data>();
    if (dataPtr) {
        dataPtr->x = 10;
        dataPtr->y = 3.14;
        std::cout << "Allocated Data: {x: " << dataPtr->x << ", y: " << dataPtr->y << "}\n";
    }

    allocator.printStatus();

    // Attempt to allocate more memory than available
    int* largeAlloc = allocator.alloc<int>(20); // Exceeds available memory
    if (!largeAlloc) {
        std::cout << "Allocation failed as expected for large request.\n";
    }

    allocator.printStatus();

    // Free all allocations and reset
    allocator.dealloc();
    allocator.dealloc();
    allocator.printStatus();

    return 0;
}
