#include "TicTacToe.hpp"
#include <iostream>
#include <iomanip>

TicTacToe::TicTacToe() : board(3, std::vector<char>(3, ' ')), currentPlayer('X') {}

void TicTacToe::printBoard() {
    std::cout << "\nCurrent Board:\n";
    for (const auto& row : board) {
        for (const auto& cell : row) {
            std::cout << std::setw(3) << (cell == ' ' ? '.' : cell);
        }
        std::cout << "\n";
    }
}

void TicTacToe::switchPlayer() {
    currentPlayer = (currentPlayer == 'X') ? 'O' : 'X';
}

void TicTacToe::makeMove() {
    int row, col;
    auto isValidMove = [&](int r, int c) {
        return r >= 0 && r < 3 && c >= 0 && c < 3 && board[r][c] == ' ';
    };

    do {
        std::cout << "Player " << currentPlayer << ", enter your move (row and column: 0, 1, 2): ";
        std::cin >> row >> col;
        if (!isValidMove(row, col)) {
            std::cout << "Invalid move! Try again.\n";
        }
    } while (!isValidMove(row, col));

    board[row][col] = currentPlayer;
}

bool TicTacToe::isWinner() {
    for (int i = 0; i < 3; ++i) {
        if ((board[i][0] == currentPlayer && board[i][1] == currentPlayer && board[i][2] == currentPlayer) ||
            (board[0][i] == currentPlayer && board[1][i] == currentPlayer && board[2][i] == currentPlayer)) {
            return true;
        }
    }

    if ((board[0][0] == currentPlayer && board[1][1] == currentPlayer && board[2][2] == currentPlayer) ||
        (board[0][2] == currentPlayer && board[1][1] == currentPlayer && board[2][0] == currentPlayer)) {
        return true;
    }

    return false;
}

bool TicTacToe::isDraw() {
    for (const auto& row : board) {
        for (const auto& cell : row) {
            if (cell == ' ') return false;
        }
    }
    return true;
}

void TicTacToe::startGame() {
    std::cout << "Welcome to Tic Tac Toe!\n";
    while (true) {
        printBoard();
        makeMove();

        if (isWinner()) {
            printBoard();
            std::cout << "Player " << currentPlayer << " wins!\n";
            break;
        } else if (isDraw()) {
            printBoard();
            std::cout << "It's a draw!\n";
            break;
        }

        switchPlayer();
    }
}
